﻿// $Id: FormSettings.cs 20 2008-03-26 02:51:32Z arcao $
//
// Copyright (C) 2008 Martin "Arcao" Sloup <arcao@arcao.com>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Windows.Forms;
using OSD.PluginBase;

namespace OSD.Program
{
    internal partial class FormSettings : Form
    {
        private readonly Plugins plugins;
        private readonly Settings settings;

        public FormSettings()
        {
            InitializeComponent();

            //uprava fontu
            Util.SetSystemFont(Controls);

            settings = Settings.getInstance();
            plugins = Plugins.GetInstance();

            LoadSettings();

            buttonDownloadPluginSettings.Enabled = false;
            if (listDownloadPlugins.Items.Count > 0)
            {
                listDownloadPlugins.SelectedIndex = 0;
            }
            textPluginUrl.Font = new Font(textPluginUrl.Font, FontStyle.Underline);
            textPluginUrl.Cursor = Cursors.Hand;
            textPluginUrl.ForeColor = SystemColors.Highlight;
        }

        #region Load / Save methods

        private void LoadSettings()
        {
            //nacteme hodnoty
            numDownloadingThreads.Value = settings.getInt("osd/settings/download_threads", 1);
            textSaveTo.Text = settings.getString("osd/settings/destination_path", @"c:\");
            numTimeoutWaiting.Value = settings.getInt("osd/settings/timeout_waiting", 300);

            checkBoxDeleteAfterDownload.Checked = settings.getBool("osd/settings/delete_after_download", false);

            comboBoxReadBufferLength.Text = settings.getInt("osd/settings/read_buffer_length", 64).ToString();

            // nacteme vsechny dostupne pluginy
            listDownloadPlugins.Items.AddRange(Plugins.GetAvailablePlugins().ToArray());

            //nyni zaskrtneme vsechny pouzite pluginy
            foreach (DownloadPlugin item in plugins)
            {
                for (int i = 0; i < listDownloadPlugins.Items.Count; i++)
                {
                    if (listDownloadPlugins.Items[i].GetType().Name == item.GetType().Name)
                        listDownloadPlugins.SetItemChecked(i, true);
                }
            }
        }

        private void SaveSettings()
        {
            settings.setInt("osd/settings/download_threads", (int) numDownloadingThreads.Value);
            settings.setString("osd/settings/destination_path", textSaveTo.Text);
            settings.setInt("osd/settings/timeout_waiting", (int) numTimeoutWaiting.Value);

            settings.setBool("osd/settings/delete_after_download", checkBoxDeleteAfterDownload.Checked);

            settings.setInt("osd/settings/read_buffer_length", int.Parse(comboBoxReadBufferLength.Text));

            plugins.Clear();
            foreach (DownloadPlugin dp in listDownloadPlugins.CheckedItems)
            {
                plugins.AddPlugin(dp);
            }
            //plugins.SetPlugins(listDownloadPlugins.CheckedItems as IEnumerable<DownloadPlugin>);
            plugins.Save();
            settings.Save();
        }

        #endregion

        private void buttonOk_Click(object sender, EventArgs e)
        {
            if (textSaveTo.Text.Trim().Length == 0 || !Directory.Exists(textSaveTo.Text))
            {
                MessageBox.Show(this,
                                "Zadaný adresář pro uložení stažených souborů neexistuje, nebo není zadán. Pro pokračování je nutné, aby byl vyplněn existující adresář.",
                                Text, MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            SaveSettings();
            DialogResult = DialogResult.OK;
            Close();
        }

        private void buttonCancel_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
            Close();
        }

        private void textPluginUrl_DoubleClick(object sender, EventArgs e)
        {
            string url = textPluginUrl.Text;
            if (url.Length > 0) Process.Start(url);
        }

        private void listDownloadPlugins_SelectedIndexChanged(object sender, EventArgs e)
        {
            DownloadPlugin dp = (DownloadPlugin) listDownloadPlugins.SelectedItem;
            textPluginName.Text = dp.Name;
            textPluginAuthor.Text = dp.Author;
            textPluginUrl.Text = dp.PluginWeb;

            buttonDownloadPluginSettings.Enabled = dp.HasSettingDialog;
        }

        private void buttonDownloadPluginSettings_Click(object sender, EventArgs e)
        {
            DownloadPlugin dp = (DownloadPlugin) listDownloadPlugins.SelectedItem;
            if (dp == null)
            {
                buttonDownloadPluginSettings.Enabled = false;
                return;
            }

            if (dp.HasSettingDialog) dp.ShowSettingDialog(this);
        }

        private void buttonSaveToBrowse_Click(object sender, EventArgs e)
        {
            saveToBrowserDialog.SelectedPath = textSaveTo.Text;
            if (saveToBrowserDialog.ShowDialog() == DialogResult.OK)
                textSaveTo.Text = saveToBrowserDialog.SelectedPath;
        }
    }
}